﻿#ifndef DX_STATIC_VARIABLES_H
#define DX_STATIC_VARIABLES_H

#include <Windows.h>
#include <d3d11.h>
#include <d3dx11.h>
#include <d3dx10.h>

#include <cassert>

#define CLIENT_WIDTH 500
#define CLIENT_HEIGHT 500

// a struct to define a single vertex
struct VERTEX
{
    FLOAT X, Y, Z;
    D3DXVECTOR3 Normal;
};

// a struct to define the constant buffer
struct CBUFFER
{
    D3DXMATRIX Final;
    D3DXMATRIX Rotation;
    D3DXVECTOR4 LightVector;
    D3DXCOLOR LightColor;
    D3DXCOLOR AmbientColor;
};

// DirectX 프로그램에서 전역변수처럼 쓰이는 것들이므로
// 해당 변수를 모아 싱글톤 클래스로 설계함.
class DX11StaticVariables
{
    DX11StaticVariables() {};
    DX11StaticVariables(const DX11StaticVariables& other);
    ~DX11StaticVariables() {};

    static DX11StaticVariables* _instance;
public:
    static DX11StaticVariables* GetInstance()
    {
        if (_instance == nullptr) _instance = new DX11StaticVariables();
        return _instance;
    }

    UINT clientWidth = CLIENT_WIDTH;
    UINT clientHeight = CLIENT_HEIGHT;

    UINT createDeviceFlags = 0;
    D3D_FEATURE_LEVEL featureLevel;
    ID3D11Device * d3dDevice = nullptr;
    ID3D11DeviceContext * d3dImmediateContext = nullptr;
    
    UINT lMSAAQuality = 0;
    bool enableMSAA = true;

    DXGI_SWAP_CHAIN_DESC swapChainDesc;

    IDXGISwapChain * swapChain = nullptr;

    ID3D11RenderTargetView * renderTargetView = nullptr;

    D3D11_TEXTURE2D_DESC depthStencilDesc;
    ID3D11Texture2D * depthStencilBuffer = nullptr;
    ID3D11DepthStencilView * depthStencilView = nullptr;

    D3D11_VIEWPORT viewport;

    ID3D11VertexShader * vertexShader = nullptr;
    ID3D11PixelShader * pixelShader = nullptr;

    ID3D11InputLayout * InputLayout = nullptr;

    ID3D11Buffer * constantBuffer = nullptr;
    ID3D11Buffer * vertexBuffer = nullptr;
    ID3D11Buffer * indexBuffer = nullptr;

    CBUFFER envBuffer;
};
DX11StaticVariables* DX11StaticVariables::_instance = nullptr;

#endif